/* Send a command to a modem, and echo the modem's response.
 *
 * This program is typically used to document a modem's status after a
 * connection, and may also be used to gather accounting statistics.
 *
 * This program may also be used to reset a modem,
 * or send an initialization string to a modem.
 *
 * This program may be freely copied and used, as long as this header,
 * the accompanying documentation, and the author's name remain intact.
 *
 * Copyright (c) 1995, Kenneth J. Hendrickson
 *	kjh@usc.edu, kjh@seas.smu.edu
 */

#define	USAGE		"Usage: %s [-c command] [-e end_string] device\n"

#define	COMMAND		"ati6\r"	/* command to feed to modem */
#define	END_STRING	"OK"		/* end of modem response */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>

#define	STRLEN		80

char	response[STRLEN+1];		/* response from modem */

#define	ALARM		5		/* modem must respond in ALARM secs */

void
no_response(i)
int	i;
{
	/* The device didn't respond within ALARM seconds. */
	if (*response)
		fputs(response, stdout);
	else
		fprintf(stdout, "No response from device.\n");
	exit(3);
}

#define	BAD_DEV		"%s: Can't open %s for reading and writing.\n"
#define	BAD_WRITE	"%s: Failure writing to %s.\n"

char	*optarg;
int	optind, opterr, optopt;

void
main(argc, argv)
int	argc;
char	**argv;
{
	char	command[STRLEN+1] = COMMAND;
	char	end_string[STRLEN+1] = END_STRING;
	FILE	*device;
	int	opt;
	int	end_len;
	int	scan = 0;

	while ((opt = getopt(argc, argv, "c:e:")) != -1) {
		switch (opt) {
			case 'c':	/* modem command string */
				strncpy(command, optarg, STRLEN-1);
				strcat(command, "\r");
				break;
			case 'e':	/* end of modem response */
				strncpy(end_string, optarg, STRLEN-1);
				break;
			case ':':	/* missing option */
			case '?':	/* unknown option character */
			default:
				fprintf(stderr, USAGE, argv[0]);
				exit(1);
				break;
		}
	}

	end_len = strlen(end_string);

	/* Was a device file specified? */
	if (!argv[optind]) {
		fprintf(stderr, USAGE, argv[0]);
		exit(1);
	}

	/* Can we read and write to the device file? */
	if ((device = fopen(argv[optind], "r+")) == NULL) {
		fprintf(stderr, BAD_DEV, argv[0], argv[optind]);
		fprintf(stderr, USAGE, argv[0]);
		exit(1);
	}

	/* Output the command to the device. */
	if (fputs(command, device) == EOF) {
		fprintf(stderr, BAD_WRITE, argv[0], argv[optind]);
		exit(1);
	}
	fflush(device);

	/* Set the alarm, in case the modem doesn't answer. */
	(void) signal(SIGALRM, no_response);
	(void) alarm(ALARM);

	/* Read the device's response, and echo to stdout. */
	while (fgets(response+scan, STRLEN-scan, device) != NULL) {
		scan = 0;
				/* THIS IS A KLUDGE!
				 * The modem appears to be putting out
				 * two \n characters instead of CR LF.
				 * Chuck one of the \n characters.
				 */
			 	if (strcmp(response, "\n\n") == 0) {
					fputc('\n', stdout);
					response[0] = '\0';
					continue;
			 	}
		/* Echo the response, up to and including end_string. */
		while (strlen(response)-scan >= end_len) {
			if (strncmp(response+scan, end_string, end_len) == 0) {
				/* We found end_string.  We're done. */
				fputs(end_string, stdout);
				fputc('\n', stdout);
				exit(0);
			} else {
				fputc(response[scan++], stdout);
			}
		}
		memmove(response, response+scan, strlen(response+scan)+1);
		scan = strlen(response);
	}
	fputs(response+scan, stdout);
	exit(1);

}

